@@ -40,6 +40,7 @@
 #include <TelepathyQt/PendingAccount>
 #include <TelepathyQt/ProfileManager>
 #include <TelepathyQt/PendingStringList>
+#include <TelepathyQt/AccountSet>
 
 #include <KLocalizedString>
 #include <KSharedConfig>
@@ -62,10 +63,12 @@ public:
     Tp::ProfileManagerPtr profileManager;
     QDialog *dialog;
     bool thingsReady;
-    QString profileName;
+    QString providerName;
     KAccountsUiPlugin::UiType type;
     Tp::AccountPtr account;
     bool reconnectRequired;
+    QString manager;
+    QString protocol;
 };
 
 KAccountsUiProvider::KAccountsUiProvider(QObject *parent)
@@ -77,13 +80,11 @@ KAccountsUiProvider::KAccountsUiProvider(QObject *parent)
 
     Tp::registerTypes();
 
-    // Start setting up the Telepathy AccountManager.
     Tp::AccountFactoryPtr  accountFactory = Tp::AccountFactory::create(QDBusConnection::sessionBus(),
                                                                        Tp::Features() << Tp::Account::FeatureCore
                                                                        << Tp::Account::FeatureCapabilities
                                                                        << Tp::Account::FeatureProtocolInfo
                                                                        << Tp::Account::FeatureProfile);
-
     d->accountManager = Tp::AccountManager::create(accountFactory);
     d->accountManager->becomeReady();
 }
@@ -103,11 +104,7 @@ void KAccountsUiProvider::init(KAccountsUiPlugin::UiType type)
 {
     d->type = type;
 
-    if (d->type == KAccountsUiPlugin::NewAccountDialog) {
-        d->profileManager = Tp::ProfileManager::create(QDBusConnection::sessionBus());
-        Tp::PendingOperation *op = d->profileManager->becomeReady(Tp::Features() << Tp::ProfileManager::FeatureFakeProfiles);
-        connect(op, SIGNAL(finished(Tp::PendingOperation*)), this, SLOT(onProfileManagerReady(Tp::PendingOperation*)));
-    } else {
+    if (d->type == KAccountsUiPlugin::ConfigureAccountDialog) {
         if (d->accountManager->isReady()) {
             Q_EMIT uiReady();
         } else {
@@ -117,6 +114,58 @@ void KAccountsUiProvider::init(KAccountsUiPlugin::UiType type)
     }
 }
 
+void KAccountsUiProvider::setProviderName(const QString &providerName)
+{
+    d->providerName = providerName;
+
+    Accounts::ServiceList list = KAccounts::accountsManager()->serviceList(QStringLiteral("IM"));
+    Accounts::Service neededService;
+    Q_FOREACH (const Accounts::Service &service, list) {
+        if (service.provider() == providerName) {
+            neededService = service;
+            break;
+        }
+    }
+
+    if (!neededService.isValid()) {
+        Q_EMIT  error(i18n("Cannot find the needed service file for %1. Please check your installation.", providerName));
+        return;
+    }
+
+    auto nodeList = neededService.domDocument().elementsByTagName(QStringLiteral("group"));
+
+    for (int i = 0; i < nodeList.size(); i++) {
+        auto attrsMap = nodeList.at(i).attributes();
+        if (attrsMap.contains(QStringLiteral("name")) && attrsMap.namedItem(QStringLiteral("name")).nodeValue() == QLatin1String("telepathy")) {
+            QDomElement settingNode = nodeList.at(i).firstChildElement(QStringLiteral("setting"));
+
+            while (!settingNode.isNull()) {
+                if (settingNode.attribute(QStringLiteral("name")) == QLatin1String("manager")) {
+                    d->manager = settingNode.text();
+                }
+                if (settingNode.attribute(QStringLiteral("name")) == QLatin1String("protocol")) {
+                    d->protocol = settingNode.text();
+                }
+
+                settingNode = settingNode.nextSiblingElement(QStringLiteral("setting"));
+            }
+
+            if (!d->manager.isEmpty() && !d->protocol.isEmpty()) {
+                break;
+            }
+        }
+    }
+
+    qDebug() << "Requested UI for manager" << d->manager << "and protocol" << d->protocol;
+
+    if (d->profileManager.isNull()) {
+        d->profileManager = Tp::ProfileManager::create(QDBusConnection::sessionBus());
+    }
+
+    Tp::PendingOperation *op = d->profileManager->becomeReady(Tp::Features() << Tp::ProfileManager::FeatureFakeProfiles);
+    connect(op, SIGNAL(finished(Tp::PendingOperation*)), this, SLOT(onProfileManagerReady(Tp::PendingOperation*)));
+}
+
 void KAccountsUiProvider::onProfileManagerReady(Tp::PendingOperation *op)
 {
     if (op && op->isError()) {
@@ -128,29 +177,32 @@ void KAccountsUiProvider::onProfileManagerReady(Tp::PendingOperation *op)
     // OR if profile name was set and profile manager is not yet ready, return.
     // If profile name is set and this returns, it will get through this again when profile manager
     // becomes ready and vice-versa.
-    if (d->profileName.isEmpty() || (d->profileManager && !d->profileManager->isReady(Tp::Features() << Tp::ProfileManager::FeatureFakeProfiles))) {
+    if (d->profileManager && !d->profileManager->isReady(Tp::Features() << Tp::ProfileManager::FeatureFakeProfiles)) {
         return;
     }
 
-    qDebug() << "Creating service for" << d->profileName;
+    qDebug() << "Creating service for" << d->providerName;
 
-    d->profile = d->profileManager->profileForService(d->profileName);
+    // KDE Talk is a bit special case so special handling needs to be added
+    QString profileService = d->providerName.mid(4);
 
-    if (d->profile.isNull()) {
-        Q_EMIT  error(i18n("To connect to this IM network, you need to install additional plugins. Please install the telepathy-haze and telepathy-gabble packages using your package manager."));
+    auto profiles = d->profileManager->profilesForProtocol(d->protocol);
+    Q_FOREACH (const Tp::ProfilePtr &profile, profiles) {
+        if (profile->cmName() == d->manager && profile->protocolName() == d->protocol && profileService == profile->serviceName()) {
+            d->profile = profile;
+            break;
+        }
+    }
+
+    if (d->profile.isNull() || profiles.isEmpty()) {
+        Q_EMIT  error(i18n("This IM Account cannot be created - a Telepathy Connection Manager named '%1' is missing or it cannot handle protocol '%2'. " \
+                           "Please try installing %1 with your package manager.", d->manager, d->protocol));
         return;
     }
 
-    d->connectionManager = Tp::ConnectionManager::create(d->profile->cmName());
-    connect(d->connectionManager->becomeReady(), SIGNAL(finished(Tp::PendingOperation*)),
-            this, SLOT(onConnectionManagerReady(Tp::PendingOperation*)));
-}
-
-void KAccountsUiProvider::setProviderName(const QString &providerName)
-{
-    d->profileName = providerName;
-
-    onProfileManagerReady(0);
+    d->connectionManager = Tp::ConnectionManager::create(d->manager);
+    Tp::PendingOperation *cmReady = d->connectionManager->becomeReady(Tp::Features() << Tp::ConnectionManager::FeatureCore);
+    connect(cmReady, SIGNAL(finished(Tp::PendingOperation*)), this, SLOT(onConnectionManagerReady(Tp::PendingOperation*)));
 }
 
 void KAccountsUiProvider::onConnectionManagerReady(Tp::PendingOperation*)
@@ -158,17 +210,11 @@ void KAccountsUiProvider::onConnectionManagerReady(Tp::PendingOperation*)
     Tp::ProtocolInfo protocolInfo = d->connectionManager->protocol(d->profile->protocolName());
     Tp::ProtocolParameterList parameters = protocolInfo.parameters();
 
+    d->dialog = new QDialog();
     // Add the parameters to the model.
-    ParameterEditModel *parameterModel = new ParameterEditModel(this);
+    ParameterEditModel *parameterModel = new ParameterEditModel(d->dialog);
     parameterModel->addItems(parameters, d->profile->parameters());
 
-    // Delete account previous widget if it already existed.
-    if (d->accountEditWidget) {
-        d->accountEditWidget->deleteLater();
-        d->accountEditWidget = 0;
-    }
-
-    d->dialog = new QDialog();
     d->dialog->setAttribute(Qt::WA_DeleteOnClose);
     QVBoxLayout *mainLayout = new QVBoxLayout(d->dialog);
     d->dialog->setLayout(mainLayout);
@@ -203,16 +249,16 @@ void KAccountsUiProvider::showNewAccountDialog()
 
 void KAccountsUiProvider::showConfigureAccountDialog(const quint32 accountId)
 {
-    KSharedConfigPtr kaccountsConfig = KSharedConfig::openConfig(QStringLiteral("kaccounts-ktprc"));
-    KConfigGroup ktpKaccountsGroup = kaccountsConfig->group(QStringLiteral("kaccounts-ktp"));
-    QString accountUid = ktpKaccountsGroup.readEntry(QString::number(accountId));
-
-    if (accountUid.isEmpty()) {
-        qWarning() << "Empty accountUid, returning...";
-        return;
+    Q_FOREACH (const Tp::AccountPtr &account, d->accountManager->validAccounts()->accounts())  {
+        if (account->uniqueIdentifier().endsWith(QString::number(accountId))) {
+            d->account = account;
+        }
     }
 
-    d->account = d->accountManager->accountForObjectPath(accountUid);
+    if (!d->account) {
+        qWarning() << "Couldn't find account for" << accountId;
+        return;
+    }
 
     // Get the protocol's parameters and values.
     Tp::ProtocolInfo protocolInfo = d->account->protocolInfo();
@@ -300,16 +346,10 @@ void KAccountsUiProvider::onCreateAccountDialogAccepted()
         values.remove(QLatin1String("password"));
     }
 
-    d->accountEditWidget->updateDisplayName();
-    Tp::PendingAccount *pa = d->accountManager->createAccount(d->profile->cmName(),
-                                                              d->profile->protocolName(),
-                                                              d->accountEditWidget->displayName(),
-                                                              values,
-                                                              properties);
+    //TODO: prefix all the values with telepathy or some mc-key
+    Q_EMIT success(values[QStringLiteral("account")].toString(), values[QStringLiteral("password")].toString(), values);
 
-    connect(pa,
-            SIGNAL(finished(Tp::PendingOperation*)),
-            SLOT(onAccountCreated(Tp::PendingOperation*)));
+    d->dialog->accept();
 }
 
 void KAccountsUiProvider::onCreateAccountDialogRejected()
@@ -317,45 +357,6 @@ void KAccountsUiProvider::onCreateAccountDialogRejected()
     Q_EMIT error(QString());
 }
 
-void KAccountsUiProvider::onAccountCreated(Tp::PendingOperation *op)
-{
-    if (op->isError()) {
-        Q_EMIT feedbackMessage(i18n("Failed to create account"),
-                                i18n("Possibly not all required fields are valid"),
-                                KMessageWidget::Error);
-        qWarning() << "Adding Account failed:" << op->errorName() << op->errorMessage();
-        Q_EMIT error(op->errorMessage());
-        return;
-    }
-
-    // Get the PendingAccount.
-    Tp::PendingAccount *pendingAccount = qobject_cast<Tp::PendingAccount*>(op);
-    if (!pendingAccount) {
-        Q_EMIT feedbackMessage(i18n("Something went wrong with Telepathy"),
-                                QString(),
-                                KMessageWidget::Error);
-        qWarning() << "Method called with wrong type.";
-        Q_EMIT error(QStringLiteral("Something went wrong with Telepathy"));
-        return;
-    }
-
-    Tp::AccountPtr account = pendingAccount->account();
-    account->setServiceName(d->profile->serviceName());
-    if (d->accountEditWidget->connectOnAdd()) {
-        account->setRequestedPresence(Tp::Presence::available());
-    }
-
-    QVariantMap values  = d->accountEditWidget->parametersSet();
-
-    QVariantMap additionalData;
-
-    additionalData.insert(QStringLiteral("uid"), account->objectPath());
-
-    Q_EMIT success(values[QStringLiteral("account")].toString(), values[QStringLiteral("password")].toString(), additionalData);
-
-    d->dialog->accept();
-}
-
 void KAccountsUiProvider::onConfigureAccountDialogAccepted()
 {
     QVariantMap setParameters = d->accountEditWidget->parametersSet();
@@ -384,7 +385,7 @@ void KAccountsUiProvider::onConfigureAccountDialogAccepted()
 
         Q_ASSERT(psl);
         if (!psl) {
-            qWarning() << "Something  weird happened";
+            qWarning() << "Something  weird happened; couldn't update the parameters";
         }
 
         if (psl->result().size() > 0) {
